@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2009-2013 Petri Lehtinen <petri@digip.org>
  * Copyright (c) 2011-2012 Basile Starynkevitch <basile@starynkevitch.net>
+ * Copyright (c) 2015 Con Kolivas <kernel@kolivas.org>
  *
  * Jansson is free software; you can redistribute it and/or modify it
  * under the terms of the MIT license. See LICENSE for details.
@@ -24,12 +25,13 @@ void *jsonp_malloc(size_t size)
     return (*do_malloc)(size);
 }
 
-void jsonp_free(void *ptr)
+void _jsonp_free(void **ptr)
 {
-    if(!ptr)
+    if(!*ptr)
         return;
 
-    (*do_free)(ptr);
+    (*do_free)(*ptr);
+    *ptr = NULL;
 }
 
 char *jsonp_strdup(const char *str)
@@ -49,6 +51,24 @@ char *jsonp_strdup(const char *str)
     return new_str;
 }
 
+char *jsonp_strsteal(strbuffer_t *strbuff)
+{
+	size_t len = strbuff->length + 1;
+	char *ret = realloc(strbuff->value, len);
+
+	return ret;
+}
+
+char *jsonp_eolstrsteal(strbuffer_t *strbuff)
+{
+	size_t len = strbuff->length + 2;
+	char *ret = realloc(strbuff->value, len);
+
+	ret[strbuff->length] = '\n';
+	ret[strbuff->length + 1] = '\0';
+	return ret;
+}
+
 void json_set_alloc_funcs(json_malloc_t malloc_fn, json_free_t free_fn)
 {
     do_malloc = malloc_fn;
