@@ -39,8 +39,7 @@ static void bab_detect(__maybe_unused bool hotplug)
 #define BAB_SPI_BUS 0
 #define BAB_SPI_CHIP 0
 
-//#define BAB_SPI_SPEED 96000
-#define BAB_SPI_SPEED 1000000
+#define BAB_SPI_SPEED 96000
 #define BAB_SPI_BUFSIZ 1024
 
 #define BAB_DELAY_USECS 0
@@ -145,8 +144,9 @@ static const uint32_t bab_test_data[BAB_TEST_DATA] = {
  */
 #define BAB_MAXSPEED 57
 #define BAB_DEFMAXSPEED 55
-#define BAB_DEFSPEED 54
+#define BAB_DEFSPEED 53
 #define BAB_MINSPEED 52
+#define BAB_ABSMINSPEED 32
 
 /*
  * % of errors to tune the speed up or down
@@ -268,6 +268,7 @@ typedef struct witem {
 	struct work *work;
 	struct bab_work_send chip_input;
 	bool ci_setup;
+	bool rolled;
 	int nonces;
 	struct timeval work_start;
 } WITEM;
@@ -323,11 +324,12 @@ static double chip_speed_ranges[BAB_CHIP_SPEEDS - 1] =
 	{ 0.0, 0.8, 1.6, 2.2, 2.8 };
 // Greater than the last one above means it's the last speed
 static char *chip_speed_names[BAB_CHIP_SPEEDS] =
-	{ "Dead", "V.Slow", "Slow", "OK", "Good", "Fast" };
+	{ "Bad", "V.Slow", "Slow", "OK", "Good", "Fast" };
 
 /*
  * This is required to do chip tuning
  * If disabled, it will simply run the chips at default speed
+ * unless they never return valid results
  */
 #define UPDATE_HISTORY 1
 
@@ -346,17 +348,23 @@ struct bab_info {
 	int chips;
 	int chips_per_bank[BAB_MAXBANKS+1];
 	int missing_chips_per_bank[BAB_MAXBANKS+1];
+	int bank_first_chip[BAB_MAXBANKS+1];
+	int bank_last_chip[BAB_MAXBANKS+1];
 	int boards;
 	int banks;
 	uint32_t chip_spis[BAB_MAXCHIPS+1];
 
 	int reply_wait;
+	uint64_t reply_waits;
 
 	cgsem_t scan_work;
 	cgsem_t spi_work;
 	cgsem_t spi_reply;
 	cgsem_t process_reply;
 
+	bool disabled[BAB_MAXCHIPS];
+	int total_disabled;
+
 	struct bab_work_reply chip_results[BAB_MAXCHIPS];
 	struct bab_work_reply chip_prev[BAB_MAXCHIPS];
 
@@ -450,13 +458,16 @@ struct bab_info {
 	K_LIST *nfree_list;
 	K_STORE *good_nonces[BAB_MAXCHIPS];
 	K_STORE *bad_nonces[BAB_MAXCHIPS];
-#if UPDATE_HISTORY
+
 	struct timeval first_work[BAB_MAXCHIPS];
+#if UPDATE_HISTORY
 	uint32_t work_count[BAB_MAXCHIPS];
 	struct timeval last_tune[BAB_MAXCHIPS];
 	uint8_t bad_fast[BAB_MAXCHIPS];
-	bool dead_msg[BAB_MAXCHIPS];
+	bool bad_msg[BAB_MAXCHIPS];
 #endif
+	uint64_t work_unrolled;
+	uint64_t work_rolled;
 
 	// bab-options (in order)
 	uint8_t max_speed;
@@ -520,6 +531,34 @@ struct bab_info {
 // Don't send work more often than this
 #define BAB_EXPECTED_WORK_DELAY_mS 899
 
+/*
+ * If a chip only has bad results after this time limit in seconds,
+ * then switch it down to min_speed
+ */
+#define BAB_BAD_TO_MIN (HISTORY_TIME_S + 10)
+
+/*
+ * Also, just to be sure it's actually mining, it must have got this
+ * many bad results before considering disabling it
+ */
+#define BAB_BAD_COUNT 100
+
+/*
+ * If a chip only has bad results after this time limit in seconds,
+ * then disable it
+ * A chip only returning bad results will use a lot more CPU than
+ * an ok chip since all results will be tested against all unexpired
+ * work that's been sent to the chip
+ */
+#define BAB_BAD_DEAD (BAB_BAD_TO_MIN * 2)
+
+/*
+ * Maximum bab_queue_full() will roll work if it is allowed to
+ * Since work can somtimes (rarely) queue up with many chips,
+ * limit it to avoid it getting too much range in the pending work
+ */
+#define BAB_MAX_ROLLTIME 42
+
 static void bab_ms3steps(uint32_t *p)
 {
 	uint32_t a, b, c, d, e, f, g, h, new_e, new_a;
@@ -603,7 +642,10 @@ static void cleanup_older(struct cgpu_info *babcgpu, int chip, K_ITEM *witem)
 
 		k_unlink_item(babinfo->chip_work[chip], tail);
 		K_WUNLOCK(babinfo->chip_work[chip]);
-		work_completed(babcgpu, DATAW(tail)->work);
+		if (DATAW(tail)->rolled)
+			free_work(DATAW(tail)->work);
+		else
+			work_completed(babcgpu, DATAW(tail)->work);
 		K_WLOCK(babinfo->chip_work[chip]);
 		k_add_head(babinfo->wfree_list, tail);
 		tail = babinfo->chip_work[chip]->tail;
@@ -614,7 +656,10 @@ static void cleanup_older(struct cgpu_info *babcgpu, int chip, K_ITEM *witem)
 		while (tail && tail != witem) {
 			k_unlink_item(babinfo->chip_work[chip], tail);
 			K_WUNLOCK(babinfo->chip_work[chip]);
-			work_completed(babcgpu, DATAW(tail)->work);
+			if (DATAW(tail)->rolled)
+				free_work(DATAW(tail)->work);
+			else
+				work_completed(babcgpu, DATAW(tail)->work);
 			K_WLOCK(babinfo->chip_work[chip]);
 			k_add_head(babinfo->wfree_list, tail);
 			tail = babinfo->chip_work[chip]->tail;
@@ -667,7 +712,7 @@ static void _bab_reset(__maybe_unused struct cgpu_info *babcgpu, struct bab_info
 // TODO: handle a false return where this is called?
 static bool _bab_txrx(struct cgpu_info *babcgpu, struct bab_info *babinfo, K_ITEM *item, bool detect_ignore, const char *file, const char *func, const int line)
 {
-	int bank, i, count;
+	int bank, i, count, chip1, chip2;
 	uint32_t siz, pos;
 	struct spi_ioc_transfer tran;
 	uintptr_t rbuf, wbuf;
@@ -751,9 +796,31 @@ static bool _bab_txrx(struct cgpu_info *babcgpu, struct bab_info *babinfo, K_ITE
 		count++;
 		if (ioctl(babinfo->spifd, SPI_IOC_MESSAGE(1), (void *)&tran) < 0) {
 			if (!detect_ignore || errno != 110) {
-				applog(LOG_ERR, "%s%d: ioctl (%d) failed err=%d" BAB_FFL,
-						babcgpu->drv->name, babcgpu->device_id,
-						count, errno, BAB_FFL_PASS);
+				for (bank = BAB_MAXBANKS; bank >= 0; bank--) {
+					if (DATAS(item)->bank_off[bank] &&
+					    pos >= DATAS(item)->bank_off[bank]) {
+						break;
+					}
+				}
+				for (chip1 = babinfo->chips-1; chip1 >= 0; chip1--) {
+					if (DATAS(item)->chip_off[chip1] &&
+					    pos >= DATAS(item)->chip_off[chip1]) {
+						break;
+					}
+				}
+				for (chip2 = babinfo->chips-1; chip2 >= 0; chip2--) {
+					if (DATAS(item)->chip_off[chip2] &&
+					    (pos + tran.len) >= DATAS(item)->chip_off[chip2]) {
+						break;
+					}
+				}
+				applog(LOG_ERR, "%s%d: ioctl (%d) siz=%d bank=%d chip=%d-%d"
+						" failed err=%d" BAB_FFL,
+						babcgpu->drv->name,
+						babcgpu->device_id,
+						count, (int)(tran.len),
+						bank, chip1, chip2,
+						errno, BAB_FFL_PASS);
 			}
 			return false;
 		}
@@ -886,65 +953,66 @@ static void bab_put(struct bab_info *babinfo, K_ITEM *sitem)
 			bank = babinfo->chip_bank[i];
 			BAB_ADD_BREAK(sitem);
 		}
-		if (BAB_CFGD_SET(babinfo->chip_conf[i]) || !babinfo->chip_conf[i]) {
-			bab_set_osc(babinfo, i);
-			bab_add_data(sitem, BAB_OSC_ADDR, babinfo->osc, sizeof(babinfo->osc));
-			bab_config_reg(sitem, BAB_ICLK_REG, BAB_ICLK_BIT(babinfo->chip_conf[i]));
-			bab_config_reg(sitem, BAB_FAST_REG, BAB_FAST_BIT(babinfo->chip_conf[i]));
-			bab_config_reg(sitem, BAB_DIV2_REG, BAB_DIV2_BIT(babinfo->chip_conf[i]));
-			bab_config_reg(sitem, BAB_SLOW_REG, BAB_SLOW_BIT(babinfo->chip_conf[i]));
-			bab_config_reg(sitem, BAB_OCLK_REG, BAB_OCLK_BIT(babinfo->chip_conf[i]));
-			for (reg = BAB_REG_CLR_FROM; reg <= BAB_REG_CLR_TO; reg++)
-				bab_config_reg(sitem, reg, false);
-			if (babinfo->chip_conf[i]) {
-				bab_add_data(sitem, BAB_COUNT_ADDR, bab_counters, sizeof(bab_counters));
-				bab_add_data(sitem, BAB_W1A_ADDR, bab_w1, sizeof(bab_w1));
-				bab_add_data(sitem, BAB_W1B_ADDR, bab_w1, sizeof(bab_w1)/2);
-				bab_add_data(sitem, BAB_W2_ADDR, bab_w2, sizeof(bab_w2));
-				babinfo->chip_conf[i] ^= BAB_CFGD_VAL;
-			}
-			babinfo->old_fast[i] = babinfo->chip_fast[i];
-			babinfo->old_conf[i] = babinfo->chip_conf[i];
-		} else {
-			if (babinfo->old_fast[i] != babinfo->chip_fast[i]) {
+		if (!(babinfo->disabled[i])) {
+			if (BAB_CFGD_SET(babinfo->chip_conf[i]) || !babinfo->chip_conf[i]) {
 				bab_set_osc(babinfo, i);
 				bab_add_data(sitem, BAB_OSC_ADDR, babinfo->osc, sizeof(babinfo->osc));
+				bab_config_reg(sitem, BAB_ICLK_REG, BAB_ICLK_BIT(babinfo->chip_conf[i]));
+				bab_config_reg(sitem, BAB_FAST_REG, BAB_FAST_BIT(babinfo->chip_conf[i]));
+				bab_config_reg(sitem, BAB_DIV2_REG, BAB_DIV2_BIT(babinfo->chip_conf[i]));
+				bab_config_reg(sitem, BAB_SLOW_REG, BAB_SLOW_BIT(babinfo->chip_conf[i]));
+				bab_config_reg(sitem, BAB_OCLK_REG, BAB_OCLK_BIT(babinfo->chip_conf[i]));
+				for (reg = BAB_REG_CLR_FROM; reg <= BAB_REG_CLR_TO; reg++)
+					bab_config_reg(sitem, reg, false);
+				if (babinfo->chip_conf[i]) {
+					bab_add_data(sitem, BAB_COUNT_ADDR, bab_counters, sizeof(bab_counters));
+					bab_add_data(sitem, BAB_W1A_ADDR, bab_w1, sizeof(bab_w1));
+					bab_add_data(sitem, BAB_W1B_ADDR, bab_w1, sizeof(bab_w1)/2);
+					bab_add_data(sitem, BAB_W2_ADDR, bab_w2, sizeof(bab_w2));
+					babinfo->chip_conf[i] ^= BAB_CFGD_VAL;
+				}
 				babinfo->old_fast[i] = babinfo->chip_fast[i];
-			}
-			if (babinfo->old_conf[i] != babinfo->chip_conf[i]) {
-				if (BAB_ICLK_SET(babinfo->old_conf[i]) !=
-						 BAB_ICLK_SET(babinfo->chip_conf[i]))
-					bab_config_reg(sitem, BAB_ICLK_REG,
-							BAB_ICLK_BIT(babinfo->chip_conf[i]));
-				if (BAB_FAST_SET(babinfo->old_conf[i]) !=
-						 BAB_FAST_SET(babinfo->chip_conf[i]))
-					bab_config_reg(sitem, BAB_FAST_REG,
-							BAB_FAST_BIT(babinfo->chip_conf[i]));
-				if (BAB_DIV2_SET(babinfo->old_conf[i]) !=
-						 BAB_DIV2_SET(babinfo->chip_conf[i]))
-					bab_config_reg(sitem, BAB_DIV2_REG,
-							BAB_DIV2_BIT(babinfo->chip_conf[i]));
-				if (BAB_SLOW_SET(babinfo->old_conf[i]) !=
-						 BAB_SLOW_SET(babinfo->chip_conf[i]))
-					bab_config_reg(sitem, BAB_SLOW_REG,
-							BAB_SLOW_BIT(babinfo->chip_conf[i]));
-				if (BAB_OCLK_SET(babinfo->old_conf[i]) !=
-						 BAB_OCLK_SET(babinfo->chip_conf[i]))
-					bab_config_reg(sitem, BAB_OCLK_REG,
-							BAB_OCLK_BIT(babinfo->chip_conf[i]));
 				babinfo->old_conf[i] = babinfo->chip_conf[i];
+			} else {
+				if (babinfo->old_fast[i] != babinfo->chip_fast[i]) {
+					bab_set_osc(babinfo, i);
+					bab_add_data(sitem, BAB_OSC_ADDR, babinfo->osc, sizeof(babinfo->osc));
+					babinfo->old_fast[i] = babinfo->chip_fast[i];
+				}
+				if (babinfo->old_conf[i] != babinfo->chip_conf[i]) {
+					if (BAB_ICLK_SET(babinfo->old_conf[i]) !=
+							 BAB_ICLK_SET(babinfo->chip_conf[i]))
+						bab_config_reg(sitem, BAB_ICLK_REG,
+								BAB_ICLK_BIT(babinfo->chip_conf[i]));
+					if (BAB_FAST_SET(babinfo->old_conf[i]) !=
+							 BAB_FAST_SET(babinfo->chip_conf[i]))
+						bab_config_reg(sitem, BAB_FAST_REG,
+								BAB_FAST_BIT(babinfo->chip_conf[i]));
+					if (BAB_DIV2_SET(babinfo->old_conf[i]) !=
+							 BAB_DIV2_SET(babinfo->chip_conf[i]))
+						bab_config_reg(sitem, BAB_DIV2_REG,
+								BAB_DIV2_BIT(babinfo->chip_conf[i]));
+					if (BAB_SLOW_SET(babinfo->old_conf[i]) !=
+							 BAB_SLOW_SET(babinfo->chip_conf[i]))
+						bab_config_reg(sitem, BAB_SLOW_REG,
+								BAB_SLOW_BIT(babinfo->chip_conf[i]));
+					if (BAB_OCLK_SET(babinfo->old_conf[i]) !=
+							 BAB_OCLK_SET(babinfo->chip_conf[i]))
+						bab_config_reg(sitem, BAB_OCLK_REG,
+								BAB_OCLK_BIT(babinfo->chip_conf[i]));
+					babinfo->old_conf[i] = babinfo->chip_conf[i];
+				}
 			}
+			DATAS(sitem)->chip_off[i] = DATAS(sitem)->siz + 3;
+			chip_input = &(DATAW(DATAS(sitem)->witems[i])->chip_input);
+
+			if (babinfo->chip_conf[i])
+				bab_add_data(sitem, BAB_INP_ADDR, (uint8_t *)chip_input, sizeof(*chip_input));
+
+			chip_siz = DATAS(sitem)->siz - babinfo->chip_conf[i];
+			if (chip_siz < BAB_CHIP_MIN)
+				BAB_ADD_NOOPs(sitem, BAB_CHIP_MIN - chip_siz);
 		}
-		DATAS(sitem)->chip_off[i] = DATAS(sitem)->siz + 3;
-		chip_input = &(DATAW(DATAS(sitem)->witems[i])->chip_input);
-
-		if (babinfo->chip_conf[i])
-			bab_add_data(sitem, BAB_INP_ADDR, (uint8_t *)chip_input, sizeof(*chip_input));
-
-		chip_siz = DATAS(sitem)->siz - babinfo->chip_conf[i];
-		if (chip_siz < BAB_CHIP_MIN)
-			BAB_ADD_NOOPs(sitem, BAB_CHIP_MIN - chip_siz);
-
 		BAB_ADD_ASYNC(sitem);
 	}
 	DATAS(sitem)->chip_off[i] = DATAS(sitem)->siz;
@@ -960,16 +1028,28 @@ static void bab_put(struct bab_info *babinfo, K_ITEM *sitem)
 static bool bab_get(__maybe_unused struct cgpu_info *babcgpu, struct bab_info *babinfo, struct timeval *when)
 {
 	K_ITEM *item;
+	bool delayed;
 	int i;
 
-	cgsem_mswait(&(babinfo->spi_reply), babinfo->reply_wait);
+	item = NULL;
+	delayed = false;
+	while (item == NULL) {
+		cgsem_mswait(&(babinfo->spi_reply), babinfo->reply_wait);
 
-	K_WLOCK(babinfo->spi_sent);
-	item = k_unlink_tail(babinfo->spi_sent);
-	K_WUNLOCK(babinfo->spi_sent);
+		K_WLOCK(babinfo->spi_sent);
+		item = k_unlink_tail(babinfo->spi_sent);
+		K_WUNLOCK(babinfo->spi_sent);
 
-	if (!item)
-		return false;
+		if (!item) {
+			if (!delayed) {
+				applog(LOG_WARNING, "%s%d: Delay getting work reply ...",
+							babcgpu->drv->name,
+							babcgpu->device_id);
+				delayed = true;
+				babinfo->reply_waits++;
+			}
+		}
+	}
 
 	for (i = 0; i < babinfo->chips; i++) {
 		if (babinfo->chip_conf[i] & 0x7f) {
@@ -1190,6 +1270,8 @@ static void bab_init_chips(struct cgpu_info *babcgpu, struct bab_info *babinfo)
 			bab_detect_chips(babcgpu, babinfo, 0, BAB_V1_CHIP_TEST, BAB_MAXCHIPS);
 		}
 		babinfo->chips_per_bank[BAB_V1_BANK] = babinfo->chips;
+		babinfo->bank_first_chip[BAB_V1_BANK] = 0;
+		babinfo->bank_last_chip[BAB_V1_BANK] = babinfo->chips - 1;
 		babinfo->boards = (int)((float)(babinfo->chips - 1) / BAB_BOARDCHIPS) + 1;
 		babinfo->reply_wait = BAB_REPLY_WAIT_mS * 2;
 
@@ -1217,6 +1299,10 @@ static void bab_init_chips(struct cgpu_info *babcgpu, struct bab_info *babinfo)
 			bab_detect_chips(babcgpu, babinfo, bank, babinfo->chips, babinfo->chips + BAB_BANKCHIPS);
 			new_chips = babinfo->chips - chips;
 			babinfo->chips_per_bank[bank] = new_chips;
+			if (new_chips > 0) {
+				babinfo->bank_first_chip[bank] = babinfo->chips - new_chips;
+				babinfo->bank_last_chip[bank] = babinfo->chips - 1;
+			}
 			chips = babinfo->chips;
 			if (new_chips == 0)
 				boards = 0;
@@ -1279,39 +1365,47 @@ static void bab_get_options(struct cgpu_info *babcgpu, struct bab_info *babinfo)
 			case 0:
 				if (*ptr && tolower(*ptr) != 'd') {
 					val = atoi(ptr);
-					if (!isdigit(*ptr) || val < BAB_MINSPEED || val > BAB_MAXSPEED) {
+					if (!isdigit(*ptr) || val < BAB_ABSMINSPEED || val > BAB_MAXSPEED) {
 						quit(1, "%s"INVOP"%s '%s' must be %d <= %s <= %d",
 							babcgpu->drv->dname,
 							bab_options[which],
-							ptr, BAB_MINSPEED,
+							ptr, BAB_ABSMINSPEED,
 							bab_options[which],
 							BAB_MAXSPEED);
 					}
 					babinfo->max_speed = (uint8_t)val;
+					// Adjust def,min down if they are above max specified
+					if (babinfo->def_speed > babinfo->max_speed)
+						babinfo->def_speed = babinfo->max_speed;
+					if (babinfo->min_speed > babinfo->max_speed)
+						babinfo->min_speed = babinfo->max_speed;
 				}
 				break;
 			case 1:
 				if (*ptr && tolower(*ptr) != 'd') {
 					val = atoi(ptr);
-					if (!isdigit(*ptr) || val < BAB_MINSPEED || val > babinfo->max_speed) {
+					if (!isdigit(*ptr) || val < BAB_ABSMINSPEED || val > babinfo->max_speed) {
 						quit(1, "%s"INVOP"%s '%s' must be %d <= %s <= %d",
 							babcgpu->drv->dname,
 							bab_options[which],
-							ptr, BAB_MINSPEED,
+							ptr, BAB_ABSMINSPEED,
 							bab_options[which],
 							babinfo->max_speed);
 					}
 					babinfo->def_speed = (uint8_t)val;
+					// Adjust min down if is is above def specified
+					if (babinfo->min_speed > babinfo->def_speed)
+						babinfo->min_speed = babinfo->def_speed;
 				}
 				break;
 			case 2:
 				if (*ptr && tolower(*ptr) != 'd') {
 					val = atoi(ptr);
-					if (!isdigit(*ptr) || val < BAB_MINSPEED || val > babinfo->def_speed) {
+					if (!isdigit(*ptr) || val < BAB_ABSMINSPEED || val > babinfo->def_speed) {
 						quit(1, "%s"INVOP"%s '%s' must be %d <= %s <= %d",
 							babcgpu->drv->dname,
 							bab_options[which],
-							ptr, BAB_MINSPEED,
+							ptr, BAB_ABSMINSPEED,
 							bab_options[which],
 							babinfo->def_speed);
 					}
@@ -1411,7 +1505,7 @@ static void bab_detect(bool hotplug)
 
 	babinfo->max_speed = BAB_DEFMAXSPEED;
 	babinfo->def_speed = BAB_DEFSPEED;
-	babinfo->min_speed = BAB_MINSPEED;
+	babinfo->min_speed = BAB_ABSMINSPEED;
 
 	babinfo->tune_up = BAB_TUNEUP;
 	babinfo->tune_down = BAB_TUNEDOWN;
@@ -1438,6 +1532,11 @@ static void bab_detect(bool hotplug)
 	babinfo->spi_list = k_new_store(babinfo->sfree_list);
 	babinfo->spi_sent = k_new_store(babinfo->sfree_list);
 
+	for (i = 0; i <= BAB_MAXBANKS; i++) {
+		babinfo->bank_first_chip[i] = -1;
+		babinfo->bank_last_chip[i] = -1;
+	}
+
 	bab_init_chips(babcgpu, babinfo);
 
 	if (babinfo->boards) {
@@ -1574,9 +1673,9 @@ static void *bab_spi(void *userdata)
 				k_add_head(babinfo->chip_work[chip], witem);
 #if UPDATE_HISTORY
 				babinfo->work_count[chip]++;
+#endif
 				if (babinfo->first_work[chip].tv_sec == 0)
 					memcpy(&(babinfo->first_work[chip]), &send, sizeof(send));
-#endif
 			}
 		}
 		K_WUNLOCK(babinfo->wfree_list);
@@ -1685,12 +1784,12 @@ static void process_history(struct cgpu_info *babcgpu, int chip, struct timeval
 			item = babinfo->bad_nonces[i]->tail;
 		}
 	}
-	K_WUNLOCK(babinfo->nfree_list);
-
-	// Tuning ...
 	good_nonces = babinfo->good_nonces[chip]->count;
 	bad_nonces = babinfo->bad_nonces[chip]->count;
 
+	K_WUNLOCK(babinfo->nfree_list);
+
+	// Tuning ...
 	if (tdiff(now, &(babinfo->first_work[chip])) >= HISTORY_TIME_S &&
 	    tdiff(now, &(babinfo->last_tune[chip])) >= HISTORY_TIME_S &&
 	    (good_nonces + bad_nonces) > 0) {
@@ -1698,7 +1797,7 @@ static void process_history(struct cgpu_info *babcgpu, int chip, struct timeval
 		chip_fast = babinfo->chip_fast[chip];
 
 		/*
-		 * If dead then step it down and remember the speed
+		 * If bad then step it down and remember the speed
 		 * TODO: does a speed change reset the chip? Or is there a reset?
 		 */
 		if (good_nonces == 0) {
@@ -1708,34 +1807,34 @@ static void process_history(struct cgpu_info *babcgpu, int chip, struct timeval
 
 				babinfo->chip_fast[chip]--;
 
-				applog(LOG_WARNING, "%s%d: Chip %d DEAD - speed down from %d to %d",
+				applog(LOG_WARNING, "%s%d: Chip %d BAD - speed down from %d to %d",
 						    babcgpu->drv->name, babcgpu->device_id,
 						    chip, (int)chip_fast, (int)chip_fast - 1);
 			} else {
 				/*
-				 * Permanently DEAD since we're already at the minumum speed
+				 * Permanently BAD since we're already at the minumum speed
 				 * but only getting bad nonces
 				 */
-				if (babinfo->dead_msg[chip] == false) {
-					applog(LOG_WARNING, "%s%d: Chip %d DEAD - at min speed %d",
+				if (babinfo->bad_msg[chip] == false) {
+					applog(LOG_WARNING, "%s%d: Chip %d BAD - at min speed %d",
 							    babcgpu->drv->name, babcgpu->device_id,
 							    chip, (int)chip_fast);
 
-					babinfo->dead_msg[chip] = true;
+					babinfo->bad_msg[chip] = true;
 				}
 			}
 			goto tune_over;
 		}
 
 		/*
-		 * It 'was' permanently DEAD but a good nonce came back!
+		 * It 'was' permanently BAD but a good nonce came back!
 		 */
-		if (babinfo->dead_msg[chip]) {
+		if (babinfo->bad_msg[chip]) {
 			applog(LOG_WARNING, "%s%d: Chip %d REVIVED - at speed %d",
 					    babcgpu->drv->name, babcgpu->device_id,
 					    chip, (int)chip_fast);
 
-			babinfo->dead_msg[chip] = false;
+			babinfo->bad_msg[chip] = false;
 		}
 
 		/*
@@ -1747,8 +1846,16 @@ static void process_history(struct cgpu_info *babcgpu, int chip, struct timeval
 
 		tune = (double)bad_nonces / (double)(good_nonces + bad_nonces) * 100.0;
 
-		// Tune it down if error rate is too high
-		if (tune >= babinfo->tune_down) {
+		/*
+		 * TODO: it appears some chips just get a % bad at low speeds
+		 * so we should handle them by weighting the speed reduction vs
+		 * the HW% gained from the reduction (i.e. GH/s)
+		 * Maybe handle that when they hit min_speed, then do a gradual speed
+		 * up verifying if it is really making GH/s worse or better
+		 */
+
+		// Tune it down if error rate is too high (and it's above min)
+		if (tune >= babinfo->tune_down && chip_fast > babinfo->min_speed) {
 			babinfo->chip_fast[chip]--;
 
 			applog(LOG_WARNING, "%s%d: Chip %d High errors %.2f%% - speed down %d to %d",
@@ -1913,6 +2020,22 @@ static K_ITEM *process_nonce(struct thr_info *thr, struct cgpu_info *babcgpu, K_
 		if (babinfo->chip_max_bad[chip] < babinfo->chip_cont_bad[chip])
 			babinfo->chip_max_bad[chip] = babinfo->chip_cont_bad[chip];
 
+		// Handle chips with only bad results
+		if (babinfo->disabled[chip] == false &&
+		    babinfo->chip_good[chip] == 0 &&
+		    babinfo->chip_bad[chip] >= BAB_BAD_COUNT &&
+		    tdiff(&now, &(babinfo->first_work[chip])) >= BAB_BAD_TO_MIN) {
+			if (babinfo->chip_fast[chip] > babinfo->min_speed)
+				babinfo->chip_fast[chip] = babinfo->min_speed;
+			else if (tdiff(&now, &(babinfo->first_work[chip])) > BAB_BAD_DEAD) {
+				babinfo->disabled[chip] = true;
+				babinfo->total_disabled++;
+				applog(LOG_ERR, "%s%i: chip %d disabled!",
+						babcgpu->drv->name,
+						babcgpu->device_id,
+						chip);
+			}
+		}
 #if UPDATE_HISTORY
 		process_history(babcgpu, chip, &(DATAR(ritem)->when), false, &now);
 #endif
@@ -2023,48 +2146,54 @@ static bool bab_do_work(struct cgpu_info *babcgpu)
 	K_WUNLOCK(babinfo->sfree_list);
 
 	for (chip = 0; chip < babinfo->chips; chip++) {
-		// TODO: ignore stale work
-		K_WLOCK(babinfo->available_work);
-		witem = k_unlink_tail(babinfo->available_work);
-		K_WUNLOCK(babinfo->available_work);
-		if (!witem) {
-			applog(LOG_ERR, "%s%i: short work list (%d) expected %d - reset",
-					babcgpu->drv->name, babcgpu->device_id,
-					chip, babinfo->chips);
-
-			// Put them back in the order they were taken
+		if (!(babinfo->disabled[chip])) {
+			// TODO: ignore stale work
 			K_WLOCK(babinfo->available_work);
-			for (j = chip-1; j >= 0; j--) {
-				witem = DATAS(sitem)->witems[j];
-				k_add_tail(babinfo->available_work, witem);
-			}
+			witem = k_unlink_tail(babinfo->available_work);
 			K_WUNLOCK(babinfo->available_work);
+			if (!witem) {
+				applog(LOG_ERR, "%s%i: short work list (%d) %d expected %d - reset",
+						babcgpu->drv->name, babcgpu->device_id,
+						chip, work_items,
+						babinfo->chips - babinfo->total_disabled);
 
-			K_WLOCK(babinfo->sfree_list);
-			k_add_head(babinfo->sfree_list, sitem);
-			K_WUNLOCK(babinfo->sfree_list);
+				// Put them back in the order they were taken
+				K_WLOCK(babinfo->available_work);
+				for (j = chip-1; j >= 0; j--) {
+					witem = DATAS(sitem)->witems[j];
+					if (witem)
+						k_add_tail(babinfo->available_work, witem);
+				}
+				K_WUNLOCK(babinfo->available_work);
 
-			return false;
+				K_WLOCK(babinfo->sfree_list);
+				k_add_head(babinfo->sfree_list, sitem);
+				K_WUNLOCK(babinfo->sfree_list);
+
+				return false;
+			}
+
+			/*
+			 * TODO: do this when we get work except on LP?
+			 * (not LP so we only do ms3steps for work required)
+			 * Though that may more likely trigger the applog(short work list) above?
+			 */
+			if (DATAW(witem)->ci_setup == false) {
+				memcpy((void *)&(DATAW(witem)->chip_input.midstate[0]),
+					DATAW(witem)->work->midstate,
+					sizeof(DATAW(witem)->work->midstate));
+				memcpy((void *)&(DATAW(witem)->chip_input.merkle7),
+					(void *)&(DATAW(witem)->work->data[WORK_MERKLE7]),
+					MERKLE_BYTES);
+
+				bab_ms3steps((void *)&(DATAW(witem)->chip_input));
+
+				DATAW(witem)->ci_setup = true;
+			}
+
+			DATAS(sitem)->witems[chip] = witem;
+			work_items++;
 		}
-
-		/*
-		 * TODO: do this when we get work except on LP?
-		 * (not LP so we only do ms3steps for work required)
-		 * Though that may more likely trigger the applog(short work list) above?
-		 */
-		if (DATAW(witem)->ci_setup == false) {
-			memcpy((void *)&(DATAW(witem)->chip_input.midstate[0]),
-				DATAW(witem)->work->midstate, sizeof(DATAW(witem)->work->midstate));
-			memcpy((void *)&(DATAW(witem)->chip_input.merkle7),
-				(void *)&(DATAW(witem)->work->data[WORK_MERKLE7]), MERKLE_BYTES);
-
-			bab_ms3steps((void *)&(DATAW(witem)->chip_input));
-
-			DATAW(witem)->ci_setup = true;
-		}
-
-		DATAS(sitem)->witems[chip] = witem;
-		work_items++;
 	}
 
 	// Send
@@ -2082,54 +2211,57 @@ static bool bab_do_work(struct cgpu_info *babcgpu)
 			  babcgpu->drv->name, babcgpu->device_id);
 
 	for (chip = 0; chip < babinfo->chips; chip++) {
-		K_WLOCK(babinfo->rfree_list);
-		ritem = k_unlink_head(babinfo->rfree_list);
-		K_WUNLOCK(babinfo->rfree_list);
+		if (!(babinfo->disabled[chip])) {
+			K_WLOCK(babinfo->rfree_list);
+			ritem = k_unlink_head(babinfo->rfree_list);
+			K_WUNLOCK(babinfo->rfree_list);
 
-		DATAR(ritem)->chip = chip;
-		DATAR(ritem)->not_first_reply = babinfo->not_first_reply[chip];
-		memcpy(&(DATAR(ritem)->when), &when, sizeof(when));
+			DATAR(ritem)->chip = chip;
+			DATAR(ritem)->not_first_reply = babinfo->not_first_reply[chip];
+			memcpy(&(DATAR(ritem)->when), &when, sizeof(when));
 
-		spichk = babinfo->chip_results[chip].spichk;
-		if (spichk != 0 && spichk != 0xffffffff) {
-			babinfo->chip_spie[chip]++;
-			spie++;
-		}
-
-		nonces = 0;
-		for (rep = 0; rep < BAB_REPLY_NONCES; rep++) {
-			nonce = babinfo->chip_results[chip].nonce[rep];
-			if (nonce != babinfo->chip_prev[chip].nonce[rep]) {
-				if ((nonce & BAB_EVIL_MASK) == BAB_EVIL_NONCE)
-					babinfo->discarded_e0s++;
-				else
-					DATAR(ritem)->nonce[nonces++] = nonce;
+			spichk = babinfo->chip_results[chip].spichk;
+			if (spichk != 0 && spichk != 0xffffffff) {
+				babinfo->chip_spie[chip]++;
+				spie++;
+				// Test the results anyway
 			}
+
+			nonces = 0;
+			for (rep = 0; rep < BAB_REPLY_NONCES; rep++) {
+				nonce = babinfo->chip_results[chip].nonce[rep];
+				if (nonce != babinfo->chip_prev[chip].nonce[rep]) {
+					if ((nonce & BAB_EVIL_MASK) == BAB_EVIL_NONCE)
+						babinfo->discarded_e0s++;
+					else
+						DATAR(ritem)->nonce[nonces++] = nonce;
+				}
+			}
+
+			if (nonces == BAB_REPLY_NONCES) {
+				babinfo->chip_miso[chip]++;
+				miso++;
+				// Test the results anyway
+			}
+
+			/*
+			 * Send even with zero nonces
+			 * so cleanup_older() is called for the chip
+			 */
+			DATAR(ritem)->nonces = nonces;
+			K_WLOCK(babinfo->res_list);
+			k_add_head(babinfo->res_list, ritem);
+			K_WUNLOCK(babinfo->res_list);
+
+			cgsem_post(&(babinfo->process_reply));
+
+			babinfo->not_first_reply[chip] = true;
+
+			memcpy((void *)(&(babinfo->chip_prev[chip])),
+				(void *)(&(babinfo->chip_results[chip])),
+				sizeof(struct bab_work_reply));
 		}
 
-		if (nonces == BAB_REPLY_NONCES) {
-			babinfo->chip_miso[chip]++;
-			miso++;
-			// Test the results anyway
-		}
-
-		/*
-		 * Send even with zero nonces
-		 * so cleanup_older() is called for the chip
-		 */
-		DATAR(ritem)->nonces = nonces;
-		K_WLOCK(babinfo->res_list);
-		k_add_head(babinfo->res_list, ritem);
-		K_WUNLOCK(babinfo->res_list);
-
-		cgsem_post(&(babinfo->process_reply));
-
-		babinfo->not_first_reply[chip] = true;
-
-		memcpy((void *)(&(babinfo->chip_prev[chip])),
-			(void *)(&(babinfo->chip_results[chip])),
-			sizeof(struct bab_work_reply));
-
 	}
 
 	applog(LOG_DEBUG, "Work: items:%d spie:%d miso:%d", work_items, spie, miso);
@@ -2183,30 +2315,52 @@ static void bab_shutdown(struct thr_info *thr)
 static bool bab_queue_full(struct cgpu_info *babcgpu)
 {
 	struct bab_info *babinfo = (struct bab_info *)(babcgpu->device_data);
-	struct work *work;
+	int roll, roll_limit = BAB_MAX_ROLLTIME;
+	struct work *work, *usework;
 	K_ITEM *item;
-	int count;
-	bool ret;
+	int count, need;
+	bool ret, rolled;
 
 	K_RLOCK(babinfo->available_work);
 	count = babinfo->available_work->count;
 	K_RUNLOCK(babinfo->available_work);
 
-	if (count >= babinfo->chips)
+	if (count >= (babinfo->chips - babinfo->total_disabled))
 		ret = true;
 	else {
+		need = (babinfo->chips - babinfo->total_disabled) - count;
 		work = get_queued(babcgpu);
 		if (work) {
-			K_WLOCK(babinfo->wfree_list);
-			item = k_unlink_head_zero(babinfo->wfree_list);
-			DATAW(item)->work = work;
-			k_add_head(babinfo->available_work, item);
-			K_WUNLOCK(babinfo->wfree_list);
-		} else
+			if (roll_limit > work->drv_rolllimit)
+				roll_limit = work->drv_rolllimit;
+			roll = 0;
+			do {
+				if (roll == 0) {
+					usework = work;
+					babinfo->work_unrolled++;
+					rolled = false;
+				} else {
+					usework = copy_work_noffset(work, roll);
+					babinfo->work_rolled++;
+					rolled = true;
+				}
+
+				K_WLOCK(babinfo->wfree_list);
+				item = k_unlink_head_zero(babinfo->wfree_list);
+				DATAW(item)->work = usework;
+				DATAW(item)->rolled = rolled;
+				k_add_head(babinfo->available_work, item);
+				K_WUNLOCK(babinfo->wfree_list);
+			} while (--need > 0 && ++roll <= roll_limit);
+		} else {
 			// Avoid a hard loop when we can't get work fast enough
 			cgsleep_us(42);
+		}
 
-		ret = false;
+		if (need > 0)
+			ret = false;
+		else
+			ret = true;
 	}
 
 	return ret;
@@ -2259,7 +2413,9 @@ static struct api_data *bab_api_stats(struct cgpu_info *babcgpu)
 	struct api_data *root = NULL;
 	char data[2048];
 	char buf[32];
-	int spi_work, chip_work, i, to, j, sp, bank, chip_off;
+	int spi_work, chip_work, sp, chip, bank, chip_off, board, last_board;
+	int i, to, j, k;
+	bool bad;
 	struct timeval now;
 	double elapsed, ghs;
 	float ghs_sum, his_ghs_tot;
@@ -2629,6 +2785,108 @@ static struct api_data *bab_api_stats(struct cgpu_info *babcgpu)
 	free(tmp);
 	tmp = NULL;
 
+	switch (babinfo->version) {
+		case 1:
+			i = j = BAB_V1_BANK;
+			break;
+		case 2:
+			i = 1;
+			j = BAB_MAXBANKS;
+			break;
+	}
+	data[0] = '\0';
+	for (bank = i; bank <= j; bank++) {
+		if (babinfo->bank_first_chip[bank] >= 0) {
+			chip = babinfo->bank_first_chip[bank];
+			to = babinfo->bank_last_chip[bank];
+			for (; chip <= to; chip += BAB_BOARDCHIPS) {
+				bad = true;
+				for (k = chip; (k <= to) && (k < (chip+BAB_BOARDCHIPS)); k++) {
+					if (history_elapsed[k] > 0) {
+						double num = history_good[k];
+						// exclude the first nonce?
+						if (elapsed_is_good[k])
+							num--;
+						ghs = num * 0xffffffffull /
+							history_elapsed[k] / 1000000000.0;
+					} else
+						ghs = 0;
+
+					if (ghs > 0.0) {
+						bad = false;
+						break;
+					}
+				}
+				if (bad) {
+					board = (int)((float)(chip - babinfo->bank_first_chip[bank]) /
+							BAB_BOARDCHIPS) + 1;
+					snprintf(buf, sizeof(buf),
+							"%s%d/%d%s",
+							data[0] ? " " : "",
+							bank, board,
+							babinfo->missing_chips_per_bank[bank] ?
+							"?" : "");
+					strcat(data, buf);
+				}
+			}
+		}
+	}
+	root = api_add_string(root, "History Bad Boards", data[0] ? data : "None", true);
+
+	data[0] = '\0';
+	for (bank = i; bank <= j; bank++) {
+		if (babinfo->bank_first_chip[bank] >= 0) {
+			to = babinfo->bank_first_chip[bank];
+			chip = babinfo->bank_last_chip[bank];
+			for (; chip >= to; chip--) {
+				bad = true;
+				if (history_elapsed[chip] > 0) {
+					double num = history_good[chip];
+					// exclude the first nonce?
+					if (elapsed_is_good[chip])
+						num--;
+					ghs = num * 0xffffffffull /
+						history_elapsed[chip] / 1000000000.0;
+				} else
+					ghs = 0;
+
+				if (ghs > 0.0)
+					break;
+			}
+			/*
+			 * The output here is: a/b+c/d
+			 * a/b is the SPI/board that starts the Bad Chain
+			 * c is the number of boards after a
+			 * d is the total number of chips in the Bad Chain
+			 * A Bad Chain is a continous set of bad chips that
+			 * finish at the end of an SPI chain of boards
+			 * This might be caused by the first board, or the cables attached
+			 * to the first board, in the Bad Chain i.e. a/b
+			 * If c is zero, it's just the last board, so it's the same as any
+			 * other board having bad chips
+			 */
+			if (chip < babinfo->bank_last_chip[bank]) {
+				board = (int)((float)(chip - babinfo->bank_first_chip[bank]) /
+						BAB_BOARDCHIPS) + 1;
+				last_board = (int)((float)(babinfo->bank_last_chip[bank] -
+						babinfo->bank_first_chip[bank]) /
+						BAB_BOARDCHIPS) + 1;
+				snprintf(buf, sizeof(buf),
+						"%s%d/%d%s+%d/%d",
+						data[0] ? " " : "",
+						bank, board,
+						babinfo->missing_chips_per_bank[bank] ?
+						"?" : "",
+						last_board - board,
+						babinfo->bank_last_chip[bank] - chip);
+				strcat(data, buf);
+			}
+		}
+	}
+	root = api_add_string(root, "History Bad Chains", data[0] ? data : "None", true);
+
+	root = api_add_int(root, "Disabled Chips", &(babinfo->total_disabled), true);
+
 	for (i = 0; i < BAB_NONCE_OFFSETS; i++) {
 		snprintf(buf, sizeof(buf), "Nonce Offset 0x%08x", bab_nonce_offsets[i]);
 		root = api_add_uint64(root, buf, &(babinfo->nonce_offset_count[i]), true);
@@ -2724,6 +2982,12 @@ static struct api_data *bab_api_stats(struct cgpu_info *babcgpu)
 	root = api_add_double(root, "Send Min", &(babinfo->send_min), true);
 	root = api_add_double(root, "Send Max", &(babinfo->send_max), true);
 
+	root = api_add_int(root, "Reply Wait", &(babinfo->reply_wait), true);
+	root = api_add_uint64(root, "Reply Waits", &(babinfo->reply_waits), true);
+
+	root = api_add_uint64(root, "Work Unrolled", &(babinfo->work_unrolled), true);
+	root = api_add_uint64(root, "Work Rolled", &(babinfo->work_rolled), true);
+
 	i = (int)(babinfo->max_speed);
 	root = api_add_int(root, bab_options[0], &i, true);
 	i = (int)(babinfo->def_speed);
@@ -2747,32 +3011,34 @@ static void bab_get_statline_before(char *buf, size_t bufsiz, struct cgpu_info *
 #if UPDATE_HISTORY
 	struct timeval now;
 	double elapsed;
-	int i, dead = 0;
+	int i, bad = 0;
 
 	cgtime(&now);
 	elapsed = tdiff(&now, &(babcgpu->dev_start_tv));
 
-	// At least get 15s of nonces before saying anything is dead
+	// At least get 15s of nonces before saying anything is bad
 	if (elapsed > 15.0) {
 		K_RLOCK(babinfo->nfree_list);
 		for (i = 0; i < babinfo->chips; i++) {
 			if (babinfo->good_nonces[i]->count == 0 &&
-			    babinfo->bad_nonces[i]->count > 1)
-				dead++;
+			     babinfo->bad_nonces[i]->count > 1)
+				bad++;
 		}
 		K_RUNLOCK(babinfo->nfree_list);
 	}
 
-	tailsprintf(buf, bufsiz, "%d.%02d.%03d D:%03d | ",
+	tailsprintf(buf, bufsiz, "%d.%02d.%03d B:%03d D:%03d",
 				 babinfo->banks,
 				 babinfo->boards,
 				 babinfo->chips,
-				 dead);
+				 bad,
+				 babinfo->total_disabled);
 #else
-	tailsprintf(buf, bufsiz, "B:%d B:%02d C:%03d | ",
+	tailsprintf(buf, bufsiz, "%d.%02d.%03d D:%03d",
 				 babinfo->banks,
 				 babinfo->boards,
-				 babinfo->chips);
+				 babinfo->chips,
+				 babinfo->total_disabled);
 #endif
 }
 #endif
