@@ -1,6 +1,6 @@
 /*
  * Copyright 2013 Andrew Smith
- * Copyright 2013-2014 Con Kolivas
+ * Copyright 2013-2015 Con Kolivas
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -101,9 +101,7 @@ static bool tolines(struct cgpu_info *bflsc, int dev, char *buf, int *lines, cha
 
 	ptr = strdup(buf);
 	while (ptr && *ptr) {
-		p_items = realloc(p_items, ++p_lines * sizeof(*p_items));
-		if (unlikely(!p_items))
-			quit(1, "Failed to realloc p_items in tolines");
+		p_items = cgrealloc(p_items, ++p_lines * sizeof(*p_items));
 		p_items[p_lines-1] = ptr;
 		ptr = strchr(ptr, '\n');
 		if (ptr)
@@ -180,9 +178,7 @@ static bool breakdown(enum breakmode mode, char *buf, int *count, char **firstna
 		comma = strchr(ptr, ',');
 		if (comma)
 			*(comma++) = '\0';
-		p_fields = realloc(p_fields, ++p_count * sizeof(*p_fields));
-		if (unlikely(!p_fields))
-			quit(1, "Failed to realloc p_fields in breakdown");
+		p_fields = cgrealloc(p_fields, ++p_count * sizeof(*p_fields));
 		p_fields[p_count-1] = ptr;
 		ptr = comma;
 	}
@@ -675,9 +671,7 @@ static bool getinfo(struct cgpu_info *bflsc, int dev)
 		goto ne;
 	}
 
-	sc_info->sc_devs = calloc(sc_info->sc_count, sizeof(struct bflsc_dev));
-	if (unlikely(!sc_info->sc_devs))
-		quit(1, "Failed to calloc in getinfo");
+	sc_info->sc_devs = cgcalloc(sc_info->sc_count, sizeof(struct bflsc_dev));
 	memcpy(&(sc_info->sc_devs[0]), &sc_dev, sizeof(sc_dev));
 	// TODO: do we care about getting this info for the rest if > 0 x-link
 
@@ -707,9 +701,7 @@ static struct cgpu_info *bflsc_detect_one(struct libusb_device *dev, struct usb_
 
 	struct cgpu_info *bflsc = usb_alloc_cgpu(&bflsc_drv, 1);
 
-	sc_info = calloc(1, sizeof(*sc_info));
-	if (unlikely(!sc_info))
-		quit(1, "Failed to calloc sc_info in bflsc_detect_one");
+	sc_info = cgcalloc(1, sizeof(*sc_info));
 	// TODO: fix ... everywhere ...
 	bflsc->device_data = (FILE *)sc_info;
 
@@ -839,7 +831,7 @@ reinit:
 
 	sc_info->scan_sleep_time = BAS_SCAN_TIME;
 	sc_info->results_sleep_time = BFLSC_RES_TIME;
-	sc_info->default_ms_work = BAS_WORK_TIME;
+	sc_info->default_ms_work = (unsigned int)BAS_WORK_TIME;
 	latency = BAS_LATENCY;
 
 	/* When getinfo() "FREQUENCY: [UNKNOWN]" is fixed -
@@ -849,20 +841,20 @@ reinit:
 	if (sc_info->sc_count > 1) {
 		newname = BFLSC_MINIRIG;
 		sc_info->scan_sleep_time = BAM_SCAN_TIME;
-		sc_info->default_ms_work = BAM_WORK_TIME;
+		sc_info->default_ms_work = (unsigned int)BAM_WORK_TIME;
 		bflsc->usbdev->ident = IDENT_BAM;
 		latency = BAM_LATENCY;
 	} else {
 		if (sc_info->sc_devs[0].engines < 34) { // 16 * 2 + 2
 			newname = BFLSC_JALAPENO;
 			sc_info->scan_sleep_time = BAJ_SCAN_TIME;
-			sc_info->default_ms_work = BAJ_WORK_TIME;
+			sc_info->default_ms_work = (unsigned int)BAJ_WORK_TIME;
 			bflsc->usbdev->ident = IDENT_BAJ;
 			latency = BAJ_LATENCY;
 		} else if (sc_info->sc_devs[0].engines < 130)  { // 16 * 8 + 2
 			newname = BFLSC_LITTLESINGLE;
 			sc_info->scan_sleep_time = BAL_SCAN_TIME;
-			sc_info->default_ms_work = BAL_WORK_TIME;
+			sc_info->default_ms_work = (unsigned int)BAL_WORK_TIME;
 			bflsc->usbdev->ident = IDENT_BAL;
 			latency = BAL_LATENCY;
 		}
@@ -872,7 +864,7 @@ reinit:
 	if (sc_info->ident == IDENT_BMA) {
 		bflsc->drv->queue_full = &bflsc28_queue_full;
 		sc_info->scan_sleep_time = BMA_SCAN_TIME;
-		sc_info->default_ms_work = BMA_WORK_TIME;
+		sc_info->default_ms_work = (unsigned int)BMA_WORK_TIME;
 		sc_info->results_sleep_time = BMA_RES_TIME;
 	}
 
@@ -1904,11 +1896,11 @@ static bool bflsc28_queue_full(struct cgpu_info *bflsc)
 		if (!field) {
 			applog(LOG_WARNING, "%s%d: Ran out of queued IDs after %d of %d",
 			       bflsc->drv->name, bflsc->device_id, i, queued);
-			queued = i - 1;
+			queued = i;
 			goto out;
 		}
 		sscanf(field, "%04x", &uid);
-		bwork = calloc(sizeof(struct bflsc_work), 1);
+		bwork = cgcalloc(sizeof(struct bflsc_work), 1);
 		bwork->id = uid;
 		bwork->work = work;
 
