@@ -181,6 +181,30 @@ retry:
 	return true;
 }
 
+static bool hfa_send_generic_frame(struct cgpu_info *hashfast, uint8_t opcode, uint8_t chip_address,
+				   uint8_t core_address, uint16_t hdata, uint8_t *data, int len)
+{
+	uint8_t packet[256];
+	struct hf_header *p = (struct hf_header *)packet;
+	int tx_length, ret, amount;
+
+	p->preamble = HF_PREAMBLE;
+	p->operation_code = opcode;
+	p->chip_address = chip_address;
+	p->core_address = core_address;
+	p->hdata = htole16(hdata);
+	p->data_length = len / 4;
+	p->crc8 = hfa_crc8(packet);
+
+	if (len)
+		memcpy(&packet[sizeof(struct hf_header)], data, len);
+	tx_length = sizeof(struct hf_header) + len;
+
+	ret = usb_write(hashfast, (char *)packet, tx_length, &amount, C_NULL);
+
+	return ((ret >= 0) && (amount == tx_length));
+}
+
 static bool hfa_send_frame(struct cgpu_info *hashfast, uint8_t opcode, uint16_t hdata,
 			   uint8_t *data, int len)
 {
@@ -370,6 +394,47 @@ static void hfa_choose_opname(struct cgpu_info *hashfast, struct hashfast_info *
 	hfa_write_opname(hashfast, info);
 }
 
+// Generic setting header
+struct hf_settings_data {
+	uint8_t revision;
+	uint8_t ref_frequency;
+	uint16_t magic;
+	uint16_t frequency0;
+	uint16_t voltage0;
+	uint16_t frequency1;
+	uint16_t voltage1;
+	uint16_t frequency2;
+	uint16_t voltage2;
+	uint16_t frequency3;
+	uint16_t voltage3;
+} __attribute__((packed,aligned(4)));
+
+static bool hfa_set_voltages(struct cgpu_info *hashfast, struct hashfast_info *info)
+{
+	struct hf_settings_data op_settings_data;
+
+	op_settings_data.revision = 1;
+	op_settings_data.ref_frequency = 25;
+	op_settings_data.magic = HFA_MAGIC_SETTINGS_VALUE;
+
+	op_settings_data.frequency0 = info->hash_clock_rate;
+	op_settings_data.voltage0 = info->hash_voltage;
+	op_settings_data.frequency1 = info->hash_clock_rate;
+	op_settings_data.voltage1 = info->hash_voltage;
+	op_settings_data.frequency2 = info->hash_clock_rate;
+	op_settings_data.voltage2 = info->hash_voltage;
+	op_settings_data.frequency3 = info->hash_clock_rate;
+	op_settings_data.voltage3 = info->hash_voltage;
+
+	hfa_send_generic_frame(hashfast, OP_SETTINGS, 0x00, 0x01, HFA_MAGIC_SETTINGS_VALUE,
+		(uint8_t *)&op_settings_data, sizeof(op_settings_data));
+	// reset the board once to switch to new voltage settings
+	hfa_send_generic_frame(hashfast, OP_POWER, 0xff, 0x00, 0x1, NULL, 0);
+	hfa_send_generic_frame(hashfast, OP_POWER, 0xff, 0x00, 0x2, NULL, 0);
+
+	return true;
+}
+
 static bool hfa_send_shutdown(struct cgpu_info *hashfast);
 
 static bool hfa_reset(struct cgpu_info *hashfast, struct hashfast_info *info)
@@ -645,7 +710,7 @@ static void hfa_set_clock(struct cgpu_info *hashfast, struct hashfast_info *info
  * to be added in the future. */
 static void hfa_check_options(struct hashfast_info *info)
 {
-	char *p, *options, *found = NULL;
+	char *p, *options, *found = NULL, *marker;
 	int maxlen, option = 0;
 
 	if (!opt_hfa_options)
@@ -688,6 +753,16 @@ static void hfa_check_options(struct hashfast_info *info)
 					break;
 				}
 				info->hash_clock_rate = lval;
+				marker = strchr(p,'@');
+				if (marker != NULL) {
+					lval = strtol(marker+1, NULL, 10);
+					if (lval < HFA_VOLTAGE_MIN || lval > HFA_VOLTAGE_MAX) {
+						applog(LOG_ERR, "Invalid core voltage %ld set with hashfast option for %s",
+						       lval, info->op_name);
+						break;
+					}
+					info->hash_voltage = lval;
+				}
 				break;
 		}
 	}
@@ -1180,6 +1255,35 @@ static void hfa_parse_notice(struct cgpu_info *hashfast, struct hf_header *h)
 	applog(LOG_NOTICE, "%s %s NOTICE: %s", hashfast->drv->name, hashfast->unique_id, d->message);
 }
 
+static void hfa_parse_settings(struct cgpu_info *hashfast, struct hf_header *h)
+{
+	struct hashfast_info *info = hashfast->device_data;
+	struct hf_settings_data *op_settings_data = (struct hf_settings_data *)(h + 1);
+
+	// Check if packet size, revision and magic are matching
+	if ((h->data_length * 4 == sizeof(struct hf_settings_data)) &&
+	    (h->core_address == 0) &&
+	    (op_settings_data->revision == 1) &&
+	    (op_settings_data->magic == HFA_MAGIC_SETTINGS_VALUE))
+	{
+		applog(LOG_NOTICE, "%s: Device settings (%dMHz@%dmV,%dMHz@%dmV,%dMHz@%dmV,%dMHz@%dmV)", hashfast->drv->name,
+			op_settings_data->frequency0, op_settings_data->voltage0,
+			op_settings_data->frequency1, op_settings_data->voltage1,
+			op_settings_data->frequency2, op_settings_data->voltage2,
+			op_settings_data->frequency3, op_settings_data->voltage3);
+		// Set voltage only when current voltage values are different
+		if ((info->hash_voltage != 0) &&
+		    ((op_settings_data->voltage0 != info->hash_voltage) ||
+		     (op_settings_data->voltage1 != info->hash_voltage) ||
+		     (op_settings_data->voltage2 != info->hash_voltage) ||
+		     (op_settings_data->voltage3 != info->hash_voltage))) {
+			applog(LOG_NOTICE, "%s: Setting default clock and voltage to %dMHz@%dmV",
+				hashfast->drv->name, info->hash_clock_rate, info->hash_voltage);
+			hfa_set_voltages(hashfast, info);
+		}
+	}
+}
+
 static void *hfa_read(void *arg)
 {
 	struct thr_info *thr = (struct thr_info *)arg;
@@ -1224,6 +1328,10 @@ static void *hfa_read(void *arg)
 			case OP_USB_NOTICE:
 				hfa_parse_notice(hashfast, h);
 				break;
+			case OP_SETTINGS:
+				hfa_parse_settings(hashfast, h);
+				break;
+			case OP_POWER:
 			case OP_PING:
 				/* Do nothing */
 				break;
@@ -1320,6 +1428,10 @@ static bool hfa_init(struct thr_info *thr)
 		}
 	}
 
+	// Read current device settings if voltage was set in options
+	if (info->hash_voltage != 0)
+		hfa_send_generic_frame(hashfast, OP_SETTINGS, 0x00, 0x00, HFA_MAGIC_SETTINGS_VALUE, NULL, 0);
+
 	mutex_init(&info->lock);
 	mutex_init(&info->rlock);
 	if (pthread_create(&info->read_thr, NULL, hfa_read, (void *)thr))
@@ -1339,7 +1451,7 @@ out:
 		hashfast->device_data = NULL;
 		usb_nodev(hashfast);
 	}
-		
+
 	return ret;
 }
 
