@@ -30,7 +30,7 @@ static char nibble[] = {
 static void hexdump(const uint8_t *p, unsigned int len)
 {
 	unsigned int i, addr;
-	unsigned int wordlen = sizeof(void*);
+	unsigned int wordlen = sizeof(unsigned int);
 	unsigned char v, line[BYTES_PER_LINE * 5];
 
 	for (addr = 0; addr < len; addr += BYTES_PER_LINE) {
