@@ -80,6 +80,8 @@
 #define OP_PING                         140         // Echo
 #define OP_CORE_MAP                     141         // Return core map
 #define OP_VERSION                      142         // Version information
+#define OP_FAN                          143         // Set Fan Speed
+#define OP_NAME                         144         // System name write/read
 #define OP_USB_DEBUG                    255
 
 // HashFast vendor and product ID's
@@ -112,6 +114,9 @@
 #define E_CORE_POWER_FAULT              17
 #define E_BAUD_TIMEOUT                  18
 #define E_ADDRESS_FAILURE               19
+#define E_IR_PROG_FAILURE               20
+#define E_MIXED_MISMATCH                21
+#define E_MIXED_TIMEOUT                 22
 
 #define U32SIZE(x)                      (sizeof(x)/sizeof(uint32_t))
 
@@ -125,6 +130,29 @@
 #define BAUD_RATE_PWRUP_6           921600
 #define BAUD_RATE_PWRUP_7          1152000
 
+// OP_WORK_RESTART hash clock change methods.
+//
+// May be issued *infrequently* by the host to adjust hash clock rate for thermal control
+// The "hdata" field, if non zero, contains adjustment instructions. Bits 15:12 of "hdata"
+// contain the adjustment type according to the following code, and bits 11:0 contain the
+// associated value. Examples:
+//     hdata = (1<<12)|550 = Set hash clock rate to 550 Mhz
+//     hdata = (4<<12)|1   = Increase hash clock rate by 1%
+//     hdata = (6<<12)     = Go back to whatever the "original" OP_USB_INIT settings were
+//
+// Finally, if 4 bytes of "data" follows the OP_WORK_RESTART header, then that data is taken
+// as a little endian bitmap, bit set = enable clock change to that die, bit clear = don't
+// change clock on that die, i.e. considered as a uint32_t, then 0x1 = die 0, 0x2 = die 1 etc.
+
+#define WR_NO_CHANGE                    0
+#define WR_CLOCK_VALUE                  1
+#define WR_MHZ_INCREASE                 2
+#define WR_MHZ_DECREASE                 3
+#define WR_PERCENT_INCREASE             4
+#define WR_PERCENT_DECREASE             5
+#define WR_REVERT                       6
+
+#define WR_COMMAND_SHIFT                12
 
 // Structure definitions, LE platforms
 
@@ -273,7 +301,7 @@ struct hf_usb_init_header {
 	uint8_t  pll_bypass:1;                  // Force PLL bypass, hash clock = ref clock
 	uint8_t  no_asic_initialization:1;      // Do not perform automatic ASIC initialization
 	uint8_t  do_atspeed_core_tests:1;       // Do core tests at speed, return second bitmap
-	uint8_t  leave_powered_down:1;          // Init USB only, leave device powered down
+	uint8_t  shed_supported:1;              // Host supports gwq status shed_count
 
 	uint16_t hash_clock;                    // Requested hash clock frequency
 
